
class LUDecomposition{

	public double[] solve(double[][] a) {

		int n=a.length;

		double[][] l=new double[n][n];
		double[][] u=new double[n][n];

		//LU decomposition
		for(int k=0;k<n;k++) {
			double alpha=1.0/a[k][k];
			for(int i=k;i<n;i++) {
				l[i][k]=a[i][k];
				u[k][i]=a[k][i]*alpha;
			}

			for(int i=k+1;i<n;i++) {
				for(int j=k+1;j<n;j++) {
					a[i][j]=a[i][j]-(l[i][k]*u[k][j]);
				}
			}
		}


		System.out.printf("LU Decomposition:\n");
		for(int i=0;i<n;i++) {
			for(int j=0;j<n;j++) {
				System.out.printf("%10f,",l[i][j]);
			}
			System.out.println();
		}
		System.out.println();

		for(int i=0;i<n;i++) {
			for(int j=0;j<n;j++) {
				System.out.printf("%10f,",u[i][j]);
			}
			System.out.println();
		}
		System.out.println();


		//forward substitution
		double[] y=new double[n];
		for(int i=0;i<n;i++) {
			y[i]=a[i][n];
			for(int j=0;j<i;j++) {
				y[i]-=l[i][j]*y[j];
			}
			y[i]/=l[i][i];
		}

		//backward substitution
		double[] x=new double[n];
		for(int i=n-1;i>=0;i--) {
			x[i]=y[i];
			for(int j=n-1;j>=i+1;j--) {
				x[i]-=u[i][j]*x[j];
			}
		}



		return x;

	}

}


public class Mat_15 {
	public static void main(String[] args) {


		/*
			double[][] a= {
					{ 1.0, 1.0, 1.0, 1.0, 10.0},
					{ 2.0, 1.0, 3.0, 2.0, 21.0},
					{ 1.0, 3.0, 2.0, 1.0, 17.0},
					{ 3.0, 2.0, 1.0, 1.0, 14.0}
			};
			*/

			/*
			double[][] a= {
					{ 6.0, 5.0, 4.0, 8.0},
					{12.0,13.0,10.0,16.0},
					{18.0,21.0,17.0,27.0}
			};
			*/
	/*
			double[][] a= {
					{1,2,4,11},
					{2,7,23,43},
					{4,13,47,85}
						};
						*/


			double[][] a= {
					{8,16,24,32,160},
					{2,7,12,17,70},
					{6,17,32,59,198},
					{7,22,46,105,291}
						};

			int n=a.length;
			double[] x=new double[n];

			for(int i=0;i<n;i++) {
				for(int j=0;j<n-1;j++) {
					System.out.format("%5.1f*x[%2d] + ",a[i][j],j);
				}
				System.out.format("%5.1f*x[%2d] = ",a[i][n-1],n-1);
				System.out.format("%5.1f\n",a[i][n]);
			}
			System.out.println();

			LUDecomposition lu=new LUDecomposition();

			x=lu.solve(a);

			System.out.println("answer:");
			for(int i=0;i<n;i++){
				System.out.format("x[%2d]=%5.1f\n",i,x[i]);
			}

	}
}
